#!/usr/bin/perl -w

$confGmetric = "/usr/bin/gmetric --tmax=4000 --dmax=4000";
$confSmartctl= "/usr/sbin/smartctl";
$confGanglia = "/etc/gmond.conf";

die "Ganglia is not installed"  if ! -r $confGanglia;

my $verbose = grep(/-v/,@ARGV);
my $dryrun  = grep(/-n/,@ARGV);

my $gmetricDest = "";

if (1)
  {
    my $ip;
    my $port;
    
    open(IN,"$confGanglia") || die "Cannot read $confGanglia";
    while (<IN>)
      {
        next if /^#/;
        $ip   = $1 if /mcast_channel +(.*)/;
        $port = $1 if /mcast_port +(.*)/;
      }
    close IN;
    
    $gmetricDest .= " -c$ip " if defined $ip;
    $gmetricDest .= " -p$port " if defined $port;
  }

exit 0 unless -r "/sys/block";

my %suppress;
my %keep;
$suppress{1} = 1; # 1=Raw_Read_Error_Rate
# 2 - ???
$suppress{3} = 1; # Spin_Up_Time
$keep{4} = 1; # 4 - Start_Stop_Count
$keep{5} = 1; # 5 - Reallocated_Sector_Ct
$suppress{6} = 1; # Read_Channel_Margin
$suppress{7} = 1; # Seek_Error_Rate
$suppress{8} = 1; # Seek_Time_Performance
$suppress{9}  = 1; # 9 - Power_On_Hours
$suppress{10} = 1; # Spin_Retry_Count
$suppress{11} = 1; # Calibration_Retry_Count
$suppress{12} = 1; # Power_Cycle_Count
$suppress{99} = 1;
$suppress{100} = 1;
$suppress{101} = 1;
$suppress{187} = 1;
$suppress{189} = 1;
$suppress{190} = 1;
$suppress{192} = 1; # Power-Off_Retract_Count
$suppress{193} = 1; # Load_Cycle_Count
$keep{194} = 1; # 194 - Temperature_Celsius
$suppress{195} = 1;
$keep{196} = 1; # 196 - Reallocated_Event_Count
$keep{197} = 1; # 197 - Current_Pending_Sector
$keep{198} = 1; # 198 - Offline_Uncorrectable
$keep{199} = 1; # 199 - UDMA_CRC_Error_Count
$suppress{200} = 1; # unknown
$suppress{201} = 1; # Soft_Read_Error_Rate
$suppress{202} = 1; # TA_Increase_Count
$suppress{203} = 1; # Run_Out_Cancel
$suppress{204} = 1; # Shock_Count_Write_Opern
$suppress{205} = 1; # Shock_Rate_Write_Opern
# 206
$suppress{207} = 1; # Spin_High_Current
$suppress{208} = 1; # Spin_Buzz
$suppress{209} = 1; # Offline_Seek_Performnce

foreach my $dev (`/bin/ls -1 /sys/block`)
  {
    chop $dev;
    next if ! -r "/sys/block/$dev/device";

    my $opt = "";

    if (-r "/sys/block/$dev/device/vendor")
      {
        my $vendor = `cat /sys/block/$dev/device/vendor`;

        print "[$vendor]\n" if $verbose;

        $opt = "-d ata" if $vendor =~ /^ATA\s+/;
      }

    my $smartData = `$confSmartctl $opt -a /dev/$dev 2>&1`;

    #print $smartData;

    my $model;
    my $serial;
    
    foreach my $entry (split(/\n/,$smartData))
      {
        print "-> $entry\n" if $verbose;
        
        $entry =~ s/^\s*//;
        $entry =~ s/\(Lifetime.*$//;

        my @entry = split(/\s+/,$entry);

        if ($entry =~ /^Device Model:\s+(.*)/)
          {
            $model = $1;
            print "Model [$model]\n" if $verbose;
          }
        
        if ($entry =~ /^Serial Number:\s+(.*)/)
          {
            $serial = $1;
            print "SN [$serial]\n" if $verbose;
          }
        
        next if (@entry != 10);
        
        my $id   = $entry[0];
        my $name = $entry[1];
        my $val  = $entry[9];
        
        next unless $id  =~ /^\d+$/;
        next unless $val =~ /^\d+$/;

        #next if $suppress{$id};
        next unless $keep{$id};
        
        $id = sprintf("%3d",$id);
        
        #print scalar @entry,$entry[0],$entry[1],"  $entry\n";

        my $devname = $dev;
        
        $devname = "$model-$serial" if (defined $model && defined $serial);

        my $cmd = "$confGmetric $gmetricDest -tint32 -n\"smart $devname $id $name\" -v$val";
        print "$cmd\n" if $verbose;
        system $cmd unless $dryrun;
     }
  }

exit 0;
#end file
